@@ -2,14 +2,14 @@
 #
 # mailer.py: send email describing a commit
 #
-# $HeadURL: http://svn.collab.net/repos/svn/branches/1.3.x/tools/hook-scripts/mailer/mailer.py $
-# $LastChangedDate: 2006-05-23 02:17:16 +0200 (Tue, 23 May 2006) $
-# $LastChangedBy: danderson $
-# $LastChangedRevision: 19769 $
+# $HeadURL: http://svn.collab.net/repos/svn/branches/1.4.x/tools/hook-scripts/mailer/mailer.py $
+# $LastChangedDate: 2006-07-27 19:44:08 +0000 (Thu, 27 Jul 2006) $
+# $LastChangedBy: dlr $
+# $LastChangedRevision: 20878 $
 #
 # USAGE: mailer.py commit      REPOS REVISION [CONFIG-FILE]
-#        mailer.py propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
-#        mailer.py propchange2 REPOS REVISION AUTHOR PROPNAME ACTION \
+#        mailer.py propchange  REPOS REVISION AUTHOR REVPROPNAME [CONFIG-FILE]
+#        mailer.py propchange2 REPOS REVISION AUTHOR REVPROPNAME ACTION \
 #                              [CONFIG-FILE]
 #        mailer.py lock        REPOS AUTHOR [CONFIG-FILE]
 #        mailer.py unlock      REPOS AUTHOR [CONFIG-FILE]
@@ -48,7 +48,7 @@ SEPARATOR = '=' * 78
 
 def main(pool, cmd, config_fname, repos_dir, cmd_args):
   ### TODO:  Sanity check the incoming args
-  
+
   if cmd == 'commit':
     revision = int(cmd_args[0])
     repos = Repository(repos_dir, revision, pool)
@@ -245,7 +245,7 @@ class StandardOutput(OutputBase):
 
 
 class PipeOutput(MailedOutput):
-  "Deliver a mail message to an MDA via a pipe."
+  "Deliver a mail message to an MTA via a pipe."
 
   def __init__(self, cfg, repos, prefix_param):
     MailedOutput.__init__(self, cfg, repos, prefix_param)
@@ -572,12 +572,34 @@ class DiffSelections:
 
 
 class DiffURLSelections:
-  def __init__(self, cfg, group):
-    self.add = cfg.get('diff_add_url', group, None)
-    self.copy = cfg.get('diff_copy_url', group, None)
-    self.delete = cfg.get('diff_delete_url', group, None)
-    self.modify = cfg.get('diff_modify_url', group, None)
+  def __init__(self, cfg, group, params):
+    self.cfg = cfg
+    self.group = group
+    self.params = params
 
+  def _get_url(self, action, repos_rev, change):
+    # The parameters for the URLs generation need to be placed in the
+    # parameters for the configuration module, otherwise we may get
+    # KeyError exceptions.
+    params = self.params.copy()
+    params['path'] = change.path and urllib.quote(change.path) or None
+    params['base_path'] = change.base_path and urllib.quote(change.base_path) or None
+    params['rev'] = repos_rev
+    params['base_rev'] = change.base_rev
+
+    return self.cfg.get("diff_%s_url" % action, self.group, params)
+
+  def get_add_url(self, repos_rev, change):
+    return self._get_url('add', repos_rev, change)
+
+  def get_copy_url(self, repos_rev, change):
+    return self._get_url('copy', repos_rev, change)
+
+  def get_delete_url(self, repos_rev, change):
+    return self._get_url('delete', repos_rev, change)
+
+  def get_modify_url(self, repos_rev, change):
+    return self._get_url('modify', repos_rev, change)
 
 def generate_content(renderer, cfg, repos, changelist, group, params, paths,
                      pool):
@@ -587,7 +609,7 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
   date = time.ctime(svn.core.secs_from_timestr(svndate, pool))
 
   diffsels = DiffSelections(cfg, group, params)
-  diffurls = DiffURLSelections(cfg, group)
+  diffurls = DiffURLSelections(cfg, group, params)
 
   show_nonmatching_paths = cfg.get('show_nonmatching_paths', group, params) \
       or 'yes'
@@ -601,9 +623,9 @@ def generate_content(renderer, cfg, repos, changelist, group, params, paths,
 
   if len(paths) != len(changelist) and show_nonmatching_paths == 'yes':
     other_diffs = DiffGenerator(changelist, paths, False, cfg, repos, date,
-                                group, params, diffsels, diffurls, pool),
+                                group, params, diffsels, diffurls, pool)
   else:
-    other_diffs = [ ]
+    other_diffs = None
 
   data = _data(
     author=repos.author,
@@ -666,23 +688,14 @@ class DiffGenerator:
     self.diffurls = diffurls
     self.pool = pool
 
+    self.diff = self.diff_url = None
+
     self.idx = 0
 
   def __nonzero__(self):
     # we always have some items
     return True
 
-  def _gen_url(self, urlstr, repos_rev, change):
-    if not len(urlstr):
-      return None
-    args = {
-      'path' : change.path and urllib.quote(change.path) or None,
-      'base_path' : change.base_path and urllib.quote(change.base_path) or None,
-      'rev' : repos_rev,
-      'base_rev' : change.base_rev,
-      }
-    return urlstr % args
-    
   def __getitem__(self, idx):
     while 1:
       if self.idx == len(self.changelist):
@@ -700,7 +713,7 @@ class DiffGenerator:
       binary = None
       singular = None
       content = None
-      
+
       # just skip directories. they have no diffs.
       if change.item_kind == svn.core.svn_node_dir:
         continue
@@ -715,10 +728,8 @@ class DiffGenerator:
         # it was delete.
         kind = 'D'
 
-        # show the diff url?
-        if self.diffurls.delete:
-          diff_url = self._gen_url(self.diffurls.delete,
-                                   self.repos.rev, change)
+        # get the diff url, if any is specified
+        diff_url = self.diffurls.get_delete_url(self.repos.rev, change)
 
         # show the diff?
         if self.diffsels.delete:
@@ -728,7 +739,7 @@ class DiffGenerator:
           label1 = '%s\t%s' % (change.base_path, self.date)
           label2 = '(empty file)'
           singular = True
-          
+
       elif change.added:
         if change.base_path and (change.base_rev != -1):
           # this file was copied.
@@ -737,10 +748,8 @@ class DiffGenerator:
           # any diff of interest?
           if change.text_changed:
 
-            # show the diff url?
-            if self.diffurls.copy:
-              diff_url = self._gen_url(self.diffurls.copy,
-                                       self.repos.rev, change)
+            # get the diff url, if any is specified
+            diff_url = self.diffurls.get_copy_url(self.repos.rev, change)
 
             # show the diff?
             if self.diffsels.copy:
@@ -755,10 +764,8 @@ class DiffGenerator:
           # the file was added.
           kind = 'A'
 
-          # show the diff url?
-          if self.diffurls.add:
-            diff_url = self._gen_url(self.diffurls.add,
-                                     self.repos.rev, change)
+          # get the diff url, if any is specified
+          diff_url = self.diffurls.get_add_url(self.repos.rev, change)
 
           # show the diff?
           if self.diffsels.add:
@@ -775,10 +782,8 @@ class DiffGenerator:
         # a simple modification.
         kind = 'M'
 
-        # show the diff url?
-        if self.diffurls.modify:
-          diff_url = self._gen_url(self.diffurls.modify,
-                                   self.repos.rev, change)
+        # get the diff url, if any is specified
+        diff_url = self.diffurls.get_modify_url(self.repos.rev, change)
 
         # show the diff?
         if self.diffsels.modify:
@@ -883,8 +888,8 @@ class TextCommitRenderer:
 
     w = self.output.write
 
-    w('Author: %s\nDate: %s\nNew Revision: %s\n\n'
-      % (data.author, data.date, data.rev))
+    w('Author: %s\nDate: %s\nNew Revision: %s\n\nLog:\n%s\n\n'
+      % (data.author, data.date, data.rev, data.log))
 
     # print summary sections
     self._render_list('Added', data.added_data)
@@ -901,12 +906,11 @@ class TextCommitRenderer:
       else:
         w('and changes in other areas\n')
 
-    w('\nLog:\n%s\n' % data.log)
-
-    self._render_diffs(data.diffs)
+    self._render_diffs(data.diffs, '')
     if data.other_diffs:
-      w('\nDiffs of changes in other areas also in this revision:\n')
-      self._render_diffs(data.other_diffs)
+      self._render_diffs(data.other_diffs,
+                         '\nDiffs of changes in other areas also'
+                         ' in this revision:\n')
 
   def _render_list(self, header, data_list):
     if not data_list:
@@ -937,34 +941,35 @@ class TextCommitRenderer:
         w('      - copied%s from r%d, %s%s\n'
           % (text, d.base_rev, d.base_path, is_dir))
 
-  def _render_diffs(self, diffs):
+  def _render_diffs(self, diffs, section_header):
+    """Render diffs. Write the SECTION_HEADER iff there are actually
+    any diffs to render."""
     w = self.output.write
+    section_header_printed = False
 
     for diff in diffs:
       if not diff.diff and not diff.diff_url:
         continue
+      if not section_header_printed:
+        w(section_header)
+        section_header_printed = True
       if diff.kind == 'D':
         w('\nDeleted: %s\n' % diff.base_path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
       elif diff.kind == 'C':
         w('\nCopied: %s (from r%d, %s)\n'
           % (diff.path, diff.base_rev, diff.base_path))
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
       elif diff.kind == 'A':
         w('\nAdded: %s\n' % diff.path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
       else:
         # kind == 'M'
         w('\nModified: %s\n' % diff.path)
-        if diff.diff_url:
-          w('Url: %s\n' % diff.diff_url)
+
+      if diff.diff_url:
+        w('URL: %s\n' % diff.diff_url)
 
       if not diff.diff:
         continue
-      
+
       w(SEPARATOR + '\n')
 
       if diff.binary:
@@ -1070,7 +1075,7 @@ class Config:
       value = getattr(sub, option, None)
     if value is None:
       value = getattr(self.defaults, option, '')
-    
+
     # parameterize it
     if params is not None:
       value = value % params
@@ -1080,6 +1085,11 @@ class Config:
     if mapper is not None:
       value = mapper(value)
 
+      # Apply any parameters that may now be available for
+      # substitution that were not before the mapping.
+      if value is not None and params is not None:
+        value = value % params
+
     return value
 
   def get_diff_cmd(self, group, args):
@@ -1226,8 +1236,8 @@ if __name__ == '__main__':
     scriptname = os.path.basename(sys.argv[0])
     sys.stderr.write(
 """USAGE: %s commit      REPOS REVISION [CONFIG-FILE]
-       %s propchange  REPOS REVISION AUTHOR PROPNAME [CONFIG-FILE]
-       %s propchange2 REPOS REVISION AUTHOR PROPNAME ACTION [CONFIG-FILE]
+       %s propchange  REPOS REVISION AUTHOR REVPROPNAME [CONFIG-FILE]
+       %s propchange2 REPOS REVISION AUTHOR REVPROPNAME ACTION [CONFIG-FILE]
        %s lock        REPOS AUTHOR [CONFIG-FILE]
        %s unlock      REPOS AUTHOR [CONFIG-FILE]
 
@@ -1257,7 +1267,7 @@ if the property was added, modified or deleted, respectively.
     usage()
 
   cmd = sys.argv[1]
-  repos_dir = sys.argv[2]
+  repos_dir = svn.core.svn_path_canonicalize(sys.argv[2])
   try:
     expected_args = cmd_list[cmd]
   except KeyError:
